@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2013 The Music Player Daemon Project
+ * Copyright (C) 2003-2014 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -24,33 +24,32 @@
 #include "util/Error.hxx"
 #include "Compiler.h"
 
+#ifdef HAVE_GLIB
 #include <glib.h>
+#endif
 
-#include <assert.h>
 #include <string.h>
 
+#ifdef HAVE_GLIB
+
 inline AllocatedPath::AllocatedPath(Donate, pointer _value)
 	:value(_value) {
 	g_free(_value);
 }
 
+#endif
+
 /* no inlining, please */
 AllocatedPath::~AllocatedPath() {}
 
-AllocatedPath
-AllocatedPath::Build(const_pointer a, const_pointer b)
-{
-	return AllocatedPath(Donate(), g_build_filename(a, b, nullptr));
-}
-
 AllocatedPath
 AllocatedPath::FromUTF8(const char *path_utf8)
 {
-	char *path = ::PathFromUTF8(path_utf8);
-	if (path == nullptr)
-		return AllocatedPath::Null();
-
-	return AllocatedPath(Donate(), path);
+#ifdef HAVE_GLIB
+	return AllocatedPath(Donate(), ::PathFromUTF8(path_utf8));
+#else
+	return FromFS(path_utf8);
+#endif
 }
 
 AllocatedPath
@@ -68,7 +67,7 @@ AllocatedPath::FromUTF8(const char *path_utf8, Error &error)
 AllocatedPath
 AllocatedPath::GetDirectoryName() const
 {
-	return AllocatedPath(Donate(), g_path_get_dirname(c_str()));
+	return FromFS(PathTraitsFS::GetParent(c_str()));
 }
 
 std::string
@@ -86,14 +85,14 @@ AllocatedPath::RelativeFS(const char *other_fs) const
 
 	other_fs += l;
 	if (*other_fs != 0) {
-		if (!PathTraits::IsSeparatorFS(*other_fs))
+		if (!PathTraitsFS::IsSeparator(*other_fs))
 			/* mismatch */
 			return nullptr;
 
 		/* skip remaining path separators */
 		do {
 			++other_fs;
-		} while (PathTraits::IsSeparatorFS(*other_fs));
+		} while (PathTraitsFS::IsSeparator(*other_fs));
 	}
 
 	return other_fs;
@@ -105,7 +104,7 @@ AllocatedPath::ChopSeparators()
 	size_t l = length();
 	const char *p = data();
 
-	while (l >= 2 && PathTraits::IsSeparatorFS(p[l - 1])) {
+	while (l >= 2 && PathTraitsFS::IsSeparator(p[l - 1])) {
 		--l;
 
 #if GCC_CHECK_VERSION(4,7) && !defined(__clang__)
