@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2013 The Music Player Daemon Project
+ * Copyright (C) 2003-2014 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -30,6 +30,28 @@
 #endif
 #endif
 
+unsigned
+MonotonicClockS(void)
+{
+#ifdef WIN32
+	return GetTickCount() / 1000;
+#elif defined(__APPLE__) /* OS X does not define CLOCK_MONOTONIC */
+	static mach_timebase_info_data_t base;
+	if (base.denom == 0)
+		(void)mach_timebase_info(&base);
+
+	return (unsigned)((mach_absolute_time() * base.numer / 1000)
+			  / (1000000 * base.denom));
+#elif defined(CLOCK_MONOTONIC)
+	struct timespec ts;
+	clock_gettime(CLOCK_MONOTONIC, &ts);
+	return ts.tv_sec;
+#else
+	/* we have no monotonic clock, fall back to time() */
+	return time(nullptr);
+#endif
+}
+
 unsigned
 MonotonicClockMS(void)
 {
@@ -40,8 +62,8 @@ MonotonicClockMS(void)
 	if (base.denom == 0)
 		(void)mach_timebase_info(&base);
 
-	return (unsigned)(((double)mach_absolute_time() * base.numer)
-			  / base.denom / 1000000);
+	return (unsigned)((mach_absolute_time() * base.numer)
+			  / (1000000 * base.denom));
 #elif defined(CLOCK_MONOTONIC)
 	struct timespec ts;
 	clock_gettime(CLOCK_MONOTONIC, &ts);
@@ -82,8 +104,8 @@ MonotonicClockUS(void)
 	if (base.denom == 0)
 		(void)mach_timebase_info(&base);
 
-	return (uint64_t)(((double)mach_absolute_time() * base.numer)
-		/ base.denom / 1000);
+	return ((uint64_t)mach_absolute_time() * (uint64_t)base.numer)
+		/ (1000 * (uint64_t)base.denom);
 #elif defined(CLOCK_MONOTONIC)
 	struct timespec ts;
 	clock_gettime(CLOCK_MONOTONIC, &ts);
@@ -96,3 +118,29 @@ MonotonicClockUS(void)
 #endif
 }
 
+#ifdef WIN32
+
+gcc_const
+static unsigned
+DeltaFileTimeS(FILETIME a, FILETIME b)
+{
+	ULARGE_INTEGER a2, b2;
+	b2.LowPart = b.dwLowDateTime;
+	b2.HighPart = b.dwHighDateTime;
+	a2.LowPart = a.dwLowDateTime;
+	a2.HighPart = a.dwHighDateTime;
+	return (a2.QuadPart - b2.QuadPart) / 10000000;
+}
+
+unsigned
+GetProcessUptimeS()
+{
+	FILETIME creation_time, exit_time, kernel_time, user_time, now;
+	GetProcessTimes(GetCurrentProcess(), &creation_time,
+			&exit_time, &kernel_time, &user_time);
+	GetSystemTimeAsFileTime(&now);
+
+	return DeltaFileTimeS(now, creation_time);
+}
+
+#endif
