@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2013 The Music Player Daemon Project
+ * Copyright (C) 2003-2014 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,7 +20,7 @@
 #include "config.h"
 #include "event/Loop.hxx"
 #include "ShutdownHandler.hxx"
-#include "ZeroconfAvahi.hxx"
+#include "zeroconf/ZeroconfAvahi.hxx"
 
 #include <stdlib.h>
 
@@ -29,7 +29,7 @@ unsigned listen_port = 1234;
 int
 main(gcc_unused int argc, gcc_unused char **argv)
 {
-	EventLoop event_loop((EventLoop::Default()));
+	EventLoop event_loop;
 	const ShutdownHandler shutdown_handler(event_loop);
 
 	AvahiInit(event_loop, "test");
