@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2013 The Music Player Daemon Project
+ * Copyright (C) 2003-2014 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -25,6 +25,14 @@
 
 #include <stdlib.h>
 
+CPPUNIT_TEST_SUITE_REGISTRATION(PcmDitherTest);
+CPPUNIT_TEST_SUITE_REGISTRATION(PcmPackTest);
+CPPUNIT_TEST_SUITE_REGISTRATION(PcmChannelsTest);
+CPPUNIT_TEST_SUITE_REGISTRATION(PcmVolumeTest);
+CPPUNIT_TEST_SUITE_REGISTRATION(PcmFormatTest);
+CPPUNIT_TEST_SUITE_REGISTRATION(PcmMixTest);
+CPPUNIT_TEST_SUITE_REGISTRATION(PcmExportTest);
+
 int
 main(gcc_unused int argc, gcc_unused char **argv)
 {
